PROGRAM SwapPT;
{ Vertauscht zwei Eintrge einer MBR-Partitionstabelle }
CONST
  MINLAUFWERK = 1;      { kleinste Laufwerksnummer }
  MAXLAUFWERK = 16;     { grte Laufwerksnummer }
  MINPAR1 = 1;          { Grenzen f. erste Nummer }
  MAXPAR1 = 3;
  MINPAR2 = 2;          { Grenzen f. zweite Nummer }
  MAXPAR2 = 4;
  SEKTORLAENGE = $0200; { Byteanzahl pro Disksektor }
  PARTOFFSET = $01BE;   { Beginn der Partitionstabelle im MBR }
  PARTLAENGE = 16;      { Byteanzahl pro Partitionseintrag }
TYPE Str2 = STRING[2];
VAR
  MBR : ARRAY[0..SEKTORLAENGE-1] OF BYTE;
  Laufwerk : Byte; E1, E2 : Integer; Test : Boolean;

FUNCTION HexByte(b : Byte): Str2;
{ Byte in String mit Hex-Zeichen wandeln }
CONST h : ARRAY[0..15] OF Char = '0123456789ABCDEF';
BEGIN
  HexByte[0] := #2;
  HexByte[1] := h[b SHR 4];
  HexByte[2] := h[b AND 15];
END;

PROCEDURE Gebrauch;
BEGIN
  Writeln('SwapPT vertauscht zwei Eintrge in der MBR-Partitionstabelle.');
  Writeln;
  Writeln('Aufruf:  swappt <laufwerksnummer> [<nummer1>-<nummer2> [test]]');
  Writeln;
  Writeln('         <laufwerksnummer>  -  ', MINLAUFWERK, '.. max.', MAXLAUFWERK, ', Nummer des Laufwerks, je nach der');
  Writeln('                                           Anzahl der vorhandenen Laufwerke');
  Writeln('         <nummer1>          -  ', MINPAR1, '..', MAXPAR1, ', Nummer des ersten Partitionseintrags');
  Writeln('         <nummer2>          -  ', MINPAR2, '..', MAXPAR2, ', Nummer des zweiten Partitionseintrags');
  Writeln;
  Writeln('Ist <laufwerksnummer> der einzige Parameter, wird nur die Partitionstabelle');
  Writeln('dargestellt.');
  Writeln('Ist als letzter Parameter das Wort "test" (oder auch nur "t") angegeben,');
  Writeln('werden nur die alte und die neue Partitionstabelle nach Vertauschung');
  Writeln('dargestellt und der genderte MBR NICHT zurck geschrieben.');
  Writeln;
  Writeln('Beispiel:');
  Writeln;
  Writeln('         swappt 2 1-4');
  Writeln;
  Writeln('Vertauscht den ersten mit dem vierten Partitionseintrag im MBR des zweiten');
  Writeln('Plattenlaufwerks.');
END;

FUNCTION HoleParameter(VAR Lw: Byte; VAR E1, E2 : Integer; VAR Tst : Boolean) : Boolean;
{ Kommandozeilen-Parameter einlesen und berprfen }
VAR f1,f2 : Integer; s : STRING;
BEGIN
{ Kein Parameter }
  IF ParamCount = 0 THEN
  BEGIN
    Gebrauch;
    HoleParameter := True;      { Abbruch }
    Exit;
  END;
{ Mindestens ein Parameter }
  Val(ParamStr(1),Lw,f1);
  IF (f1 <> 0) OR (Lw < MINLAUFWERK) OR (Lw > MAXLAUFWERK) THEN
  BEGIN
    Writeln('#### Laufwerksnummer ungltig!');
    Writeln;
    Gebrauch;
    HoleParameter := True;      { Abbruch }
    Exit;
  END;
  IF ParamCount = 1 THEN
  BEGIN
    E1 := -1;
    HoleParameter := False;     { Weiter }
    Exit;
  END;
{ Mindestens zwei Parameter }
  Tst := False;
  s := ParamStr(2);
  Val(s[1],E1,f1);
  Val(s[3],E2,f2);
  IF (s[2]<>'-') OR (f1<>0) OR (f2<>0) OR (E1<MINPAR1) OR (E1>MAXPAR1)
     OR (E2<MINPAR2) OR (E2>MAXPAR2) OR (E1>=E2) THEN
  BEGIN
    Writeln('#### Fehler im zweiten Parameter!');
    Writeln;
    Gebrauch;
    HoleParameter := True;      { Abbruch }
    Exit;
  END;
  IF (ParamCount > 2) THEN
  BEGIN
    s := ParamStr(3);
    IF Upcase(s[1]) = 'T' THEN
      Tst := True
    ELSE BEGIN
      Writeln('#### Parameter ungltig! Abbruch.');
      HoleParameter := True;
      Exit;
    END;
  END;
  HoleParameter := False;       { Weiter }
END;

FUNCTION ReadMBR (lw : Byte) : Byte; ASSEMBLER;
{ MBR von Laufwerk lw in Puffer lesen }
ASM
   mov ax,$0201
   xor ch,ch
   mov cl,1
   xor dh,dh
   mov dl,lw
   mov bx,seg MBR
   mov es,bx
   mov bx,offset MBR
   int $13
   xchg al,ah
END;

FUNCTION WriteMBR (lw : Byte) : Byte; ASSEMBLER;
{ MBR von Puffer auf Laufwerk lw schreiben }
ASM
   mov ax,$0301
   xor ch,ch
   mov cl,1
   xor dh,dh
   mov dl,lw
   mov bx,seg MBR
   mov es,bx
   mov bx,offset MBR
   int $13
   xchg al,ah
END;

FUNCTION CHS(b1,b2,b3 : Byte) : STRING;
{ CHS-Sektorangabe in String wandeln }
VAR c,h,s,st : STRING;
BEGIN
  Str((((Word(b2) AND $00C0) SHL 2) + b3), c);
  Str(b1, h);
  Str(b2 AND $3F, s);
  st := c + '/' + h + '/' + s;
  WHILE Length(st) < 11 DO
    st := ' ' + st;
  CHS := st;
END;

FUNCTION Long(b1,b2,b3,b4 : Byte) : STRING;
{ 32bit-Sektorangabe in String wandeln }
VAR l : Longint; s : STRING;
BEGIN
  l := Longint(b4) SHL 24 + Longint(b3) SHL 16 + Word(b2) SHL 8 + b1;
  Str(l,s);
  WHILE Length(s) < 9 DO
    s := ' ' + s;
  Long := s;
END;

PROCEDURE ZeigeMBR(s : STRING);
{ MBR-Partitionstabelle darstellen }
VAR e : Integer;
BEGIN
  Write('Partitionstabelle');
  IF s <> '' THEN
    Write(' ', s, ' Vertauschung');
  Writeln(':');
  Writeln;
  Writeln('      Typ   Boot  Beginn(C/H/S)  Ende(C/H/S)  Anfang(rel.Sekt.)  Lnge(Sekt.)');
  FOR e := 1 TO 4 DO
  BEGIN
    Write(e, ':  ');
    IF MBR[PARTOFFSET+4+PARTLAENGE*(e-1)] = 0 THEN
    BEGIN
      Writeln('(leer)');
      Continue;
    END;
    Write('  ', HexByte(MBR[PARTOFFSET+4+PARTLAENGE*(e-1)]), '     ');
    IF (MBR[PARTOFFSET+PARTLAENGE*(e-1)] AND $80) <> 0 THEN
      Write('*     ')
    ELSE
      Write('      ');
    Write(CHS(MBR[PARTOFFSET+1+PARTLAENGE*(e-1)],
              MBR[PARTOFFSET+2+PARTLAENGE*(e-1)],
              MBR[PARTOFFSET+3+PARTLAENGE*(e-1)]), '   ');
    Write(CHS(MBR[PARTOFFSET+5+PARTLAENGE*(e-1)],
              MBR[PARTOFFSET+6+PARTLAENGE*(e-1)],
              MBR[PARTOFFSET+7+PARTLAENGE*(e-1)]), '      ');
    Write(Long(MBR[PARTOFFSET+8+PARTLAENGE*(e-1)],
               MBR[PARTOFFSET+9+PARTLAENGE*(e-1)],
               MBR[PARTOFFSET+10+PARTLAENGE*(e-1)],
               MBR[PARTOFFSET+11+PARTLAENGE*(e-1)]), '        ');
    Write(Long(MBR[PARTOFFSET+12+PARTLAENGE*(e-1)],
               MBR[PARTOFFSET+13+PARTLAENGE*(e-1)],
               MBR[PARTOFFSET+14+PARTLAENGE*(e-1)],
               MBR[PARTOFFSET+15+PARTLAENGE*(e-1)]));
    Writeln;
  END;
  Writeln;
END;

PROCEDURE LeseMBR(lw : Byte);
{ MBR lesen + Info }
VAR Fehler : Byte;
BEGIN
  Writeln('*** Lese MBR ...');
  Fehler := ReadMBR($80+lw-1);
  IF Fehler <> 0 THEN
  BEGIN
    Writeln('#### Fehler ', HexByte(Fehler), 'h beim Lesen des MBR aufgetreten. Abbruch.');
    Halt;
  END;
  Writeln;
END;

PROCEDURE SchreibeMBR(lw : Byte);
{ MBR schreiben + Info }
VAR Fehler : Byte;
BEGIN
  Writeln('*** Schreibe MBR ...');
  Fehler := WriteMBR($80+lw-1);
  IF Fehler <> 0 THEN
  BEGIN
    Writeln('#### Fehler ', HexByte(Fehler), 'h beim Schreiben des MBR aufgetreten. Abbruch.');
    Halt;
  END;
  Writeln;
END;

PROCEDURE Vertausche (m, n : Integer);
VAR i : Integer; b : Byte;
BEGIN
  Writeln('*** Mache Vertauschung ...');
  FOR i := 0 TO PARTLAENGE-1 DO
  BEGIN
    b := MBR[PARTOFFSET+PARTLAENGE*(m-1)+i];
    MBR[PARTOFFSET+PARTLAENGE*(m-1)+i] := MBR[PARTOFFSET+PARTLAENGE*(n-1)+i];
    MBR[PARTOFFSET+PARTLAENGE*(n-1)+i] := b;
  END;
  Writeln;
END;

BEGIN
{ Hauptprogramm }
  IF HoleParameter(Laufwerk, E1, E2, Test) THEN Halt;
  LeseMBR(Laufwerk);
  IF E1 < 0 THEN
  BEGIN
    ZeigeMBR('');
    Halt;
  END;
  ZeigeMBR('VOR');
  Vertausche(E1,E2);
  ZeigeMBR('NACH');
  IF NOT Test THEN
  BEGIN
    SchreibeMBR(Laufwerk);
    Writeln('Fertig!');
  END
  ELSE
    Writeln('*** MBR nicht geschrieben.');
END.
